@@ -43,7 +43,6 @@ extern "C" {
 }
 
 #include <algorithm>
-#include <cassert>
 #include <cerrno>
 #include <cstdlib>
 #include <cstring>
@@ -63,7 +62,7 @@ extern "C" {
 #include "atf/formats.hpp"
 #include "atf/fs.hpp"
 #include "atf/io.hpp"
-#include "atf/serial.hpp"
+#include "atf/sanity.hpp"
 #include "atf/tests.hpp"
 #include "atf/text.hpp"
 #include "atf/ui.hpp"
@@ -130,7 +129,7 @@ public:
         m_sanew.sa_flags = 0;
 
         program();
-        assert(m_holders.find(m_signal) == m_holders.end());
+        PRE(m_holders.find(m_signal) == m_holders.end());
         m_holders[m_signal] = this;
     }
 
@@ -240,6 +239,24 @@ impl::vars_map::has(const std::string& key)
     return find(key) != end();
 }
 
+impl::vars_map::value_type
+impl::vars_map::parse(const std::string& str)
+{
+    if (str.empty())
+        throw std::runtime_error("-v requires a non-empty argument");
+
+    std::vector< std::string > ws = atf::text::split(str, "=");
+    if (ws.size() == 1 && str[str.length() - 1] == '=') {
+        return impl::vars_map::value_type(ws[0], "");
+    } else {
+        if (ws.size() != 2)
+            throw std::runtime_error("-v requires an argument of the form "
+                                     "var=value");
+
+        return impl::vars_map::value_type(ws[0], ws[1]);
+    }
+}
+
 // ------------------------------------------------------------------------
 // The "tcr" class.
 // ------------------------------------------------------------------------
@@ -251,10 +268,19 @@ impl::tcr::tcr(void) :
 }
 
 impl::tcr::tcr(impl::tcr::status s, const std::string& r) :
-    m_status(s),
-    m_reason(r)
+    m_status(s)
 {
-    assert(m_reason.find('\n') == std::string::npos);
+    if (r.find('\n') == std::string::npos)
+        m_reason = r;
+    else {
+        m_reason = "BOGUS REASON (THE ORIGINAL HAD NEWLINES): ";
+        for (std::string::size_type i = 0; i < r.length(); i++) {
+            if (r[i] == '\n')
+                m_reason += "<<NEWLINE>>";
+            else if (r[i] != '\r')
+                m_reason += r[i];
+        }
+    }
 }
 
 impl::tcr
@@ -325,7 +351,7 @@ impl::tc::ensure_not_empty(const std::string& name)
             ("Undefined or empty variable", name);
 
     vars_map::const_iterator iter = m_meta_data.find(name);
-    assert(iter != m_meta_data.end());
+    INV(iter != m_meta_data.end());
 
     const std::string& val = (*iter).second;
     if (val.empty())
@@ -344,7 +370,7 @@ impl::tc::get(const std::string& var)
     const
 {
     vars_map::const_iterator iter = m_meta_data.find(var);
-    assert(iter != m_meta_data.end());
+    PRE(iter != m_meta_data.end());
     return (*iter).second;
 }
 
@@ -358,10 +384,8 @@ impl::tc::get_bool(const std::string& var)
         return true;
     else if (val == "false")
         return false;
-    else {
-        assert(false);
-        return false;
-    }
+    else
+        UNREACHABLE;
 }
 
 bool
@@ -389,96 +413,94 @@ impl::tc::get_srcdir(void)
 void
 impl::tc::init(const vars_map& c, const std::string& srcdir)
 {
-    assert(m_meta_data.empty());
+    PRE(m_meta_data.empty());
 
     m_config = c; // XXX Uh, deep copy.  Should be a reference...
     m_srcdir = srcdir;
 
     m_meta_data["ident"] = m_ident;
-    m_meta_data["isolated"] = "yes";
     head();
     ensure_not_empty("descr");
     ensure_not_empty("ident");
-    ensure_boolean("isolated");
-    assert(m_meta_data["ident"] == m_ident);
-}
-
-static
-void
-enter_workdir(const impl::tc* tc, atf::fs::path& olddir,
-              atf::fs::path& workdir, const std::string& base)
-{
-    if (!tc->get_bool("isolated"))
-        workdir = atf::fs::path(base);
-    else {
-        atf::fs::path pattern = atf::fs::path(base) / "atf.XXXXXX";
-        workdir = atf::fs::create_temp_dir(pattern);
-        olddir = atf::fs::change_directory(workdir);
-    }
-}
-
-static
-void
-leave_workdir(const impl::tc* tc, atf::fs::path& olddir,
-              atf::fs::path& workdir)
-{
-    if (!tc->get_bool("isolated"))
-        return;
-
-    atf::fs::change_directory(olddir);
-    atf::fs::cleanup(workdir);
+    INV(m_meta_data["ident"] == m_ident);
 }
 
 impl::tcr
 impl::tc::safe_run(void)
     const
 {
-    tcr tcr = tcr::passed();
+    atf::fs::temp_dir workdir
+        (atf::fs::path(m_config.get("workdir")) / "atf.XXXXXX");
+    impl::tcr tcr = fork_body(workdir.get_path().str());
+    fork_cleanup(workdir.get_path().str());
+    return tcr;
+}
 
+static void
+sanitize_process(const atf::fs::path& workdir)
+{
+    // Reset all signal handlers to their default behavior.
+    struct sigaction sadfl;
+    sadfl.sa_handler = SIG_DFL;
+    sigemptyset(&sadfl.sa_mask);
+    sadfl.sa_flags = 0;
+    for (int i = 0; i < LAST_SIGNAL; i++)
+        ::sigaction(i, &sadfl, NULL);
+
+    // Reset critical environment variables to known values.
+    atf::env::set("HOME", workdir.str());
+    atf::env::unset("LANG");
+    atf::env::unset("LC_ALL");
+    atf::env::unset("LC_COLLATE");
+    atf::env::unset("LC_CTYPE");
+    atf::env::unset("LC_MESSAGES");
+    atf::env::unset("LC_MONETARY");
+    atf::env::unset("LC_NUMERIC");
+    atf::env::unset("LC_TIME");
+    atf::env::unset("TZ");
+
+    // Reset the umask.
+    ::umask(S_IWGRP | S_IWOTH);
+
+    // Set the work directory.
+    atf::fs::change_directory(workdir);
+}
+
+void
+impl::tc::check_requirements(void)
+    const
+{
     if (has("require.config")) {
         const std::string& c = get("require.config");
         std::vector< std::string > vars = text::split(c, " ");
         for (std::vector< std::string >::const_iterator iter = vars.begin();
              iter != vars.end(); iter++) {
-            if (!m_config.has(*iter)) {
-                tcr = tcr::skipped("Required configuration variable " +
+            if (!m_config.has(*iter))
+                throw tcr::skipped("Required configuration variable " +
                                    *iter + " not defined");
-                break;
-            }
         }
     }
 
-    if (tcr.get_status() == tcr::status_passed && has("require.user")) {
+    if (has("require.user")) {
         const std::string& u = get("require.user");
         if (u == "root") {
             if (!user::is_root())
-                tcr = tcr::skipped("Requires root privileges");
+                throw tcr::skipped("Requires root privileges");
         } else if (u == "unprivileged") {
             if (!user::is_unprivileged())
-                tcr = tcr::skipped("Requires an unprivileged user");
+                throw tcr::skipped("Requires an unprivileged user");
         } else
-            tcr = tcr::skipped("Invalid value in the require.user "
+            throw tcr::skipped("Invalid value in the require.user "
                                "property");
     }
 
-    if (tcr.get_status() == tcr::status_passed) {
-        // Previous checks have not made the test fail, so run it now.
-
-        fs::path olddir(".");
-        fs::path workdir(".");
-
-        try {
-            enter_workdir(this, olddir, workdir, m_config.get("workdir"));
-            tcr = fork_body(workdir.str());
-            cleanup();
-            leave_workdir(this, olddir, workdir);
-        } catch (...) {
-            leave_workdir(this, olddir, workdir);
-            throw;
-        }
+    if (has("require.progs")) {
+        std::vector< std::string > progs =
+            text::split(get("require.progs"), " ");
+        for (std::vector< std::string >::const_iterator iter =
+             progs.begin(); iter != progs.end(); iter++)
+            require_prog(*iter);
     }
-
-    return tcr;
 }
 
 impl::tcr
@@ -487,15 +509,7 @@ impl::tc::fork_body(const std::string& workdir)
 {
     tcr tcr;
 
-    // XXX To handle this correctly, we should have a RAII model to deal
-    // with temporary files.  Otherwise, there are chances that the
-    // explicit file removal at the end of this function is missed.
-    fs::path result(".");
-    if (get_bool("isolated")) {
-        result = fs::path(workdir) / "tc-result";
-    } else {
-        result = fs::create_temp_file(fs::path(workdir) / "atf.XXXXXX");
-    }
+    fs::path result(fs::path(workdir) / "tc-result");
 
     pid_t pid = ::fork();
     if (pid == -1) {
@@ -503,47 +517,13 @@ impl::tc::fork_body(const std::string& workdir)
     } else if (pid == 0) {
         int errcode;
 
-        struct sigaction sadfl;
-        sadfl.sa_handler = SIG_DFL;
-        sigemptyset(&sadfl.sa_mask);
-        sadfl.sa_flags = 0;
-        for (int i = 0; i < LAST_SIGNAL; i++)
-            ::sigaction(i, &sadfl, NULL);
-
         // Unexpected errors detected in the child process are mentioned
         // in stderr to give the user a chance to see what happened.
         // This is specially useful in those cases where these errors
         // cannot be directed to the parent process.
         try {
-            atf::env::set("HOME", workdir);
-            atf::env::unset("LANG");
-            atf::env::unset("LC_ALL");
-            atf::env::unset("LC_COLLATE");
-            atf::env::unset("LC_CTYPE");
-            atf::env::unset("LC_MESSAGES");
-            atf::env::unset("LC_MONETARY");
-            atf::env::unset("LC_NUMERIC");
-            atf::env::unset("LC_TIME");
-            atf::env::unset("TZ");
-
-            ::umask(S_IWGRP | S_IWOTH);
-
-            // This is here because require_progs can assert if the user
-            // gave bad input.  Having it here lets us capture the abort
-            // from the parent.
-            //
-            // XXX The thing is... is this appropriate?  Should we forbid
-            // the assertions in our code and use exceptions instead?
-            // Or... should all properties be handled here -- aka, in the
-            // controlled environment?
-            if (has("require.progs")) {
-                std::vector< std::string > progs =
-                    text::split(get("require.progs"), " ");
-                for (std::vector< std::string >::const_iterator iter =
-                     progs.begin(); iter != progs.end(); iter++)
-                    require_prog(*iter);
-            }
-
+            sanitize_process(atf::fs::path(workdir));
+            check_requirements();
             body();
             throw impl::tcr::passed();
         } catch (const impl::tcr& tcre) {
@@ -612,21 +592,54 @@ impl::tc::fork_body(const std::string& workdir)
                 tcr = tcr::failed("Test case received stop signal " +
                                   atf::text::to_string(WSTOPSIG(status)));
             } else
-                assert(false);
+                UNREACHABLE;
         }
     }
 
-    if (atf::fs::exists(result))
-        atf::fs::remove(result);
-
     return tcr;
 }
 
+void
+impl::tc::fork_cleanup(const std::string& workdir)
+    const
+{
+    pid_t pid = ::fork();
+    if (pid == -1) {
+        std::cerr << "WARNING: Could not fork to run test case's cleanup "
+                     "routine for " << workdir << std::endl;
+    } else if (pid == 0) {
+        int errcode = EXIT_FAILURE;
+        try {
+            sanitize_process(atf::fs::path(workdir));
+            cleanup();
+            errcode = EXIT_SUCCESS;
+        } catch (const std::exception& e) {
+            std::cerr << "WARNING: Caught unexpected exception while "
+                         "running the test case's cleanup routine: "
+                      << e.what() << std::endl;
+        } catch (...) {
+            std::cerr << "WARNING: Caught unknown exception while "
+                         "running the test case's cleanup routine"
+                      << std::endl;
+        }
+        std::exit(errcode);
+    } else {
+        int status;
+        if (::waitpid(pid, &status, 0) == -1)
+            std::cerr << "WARNING: Error while waiting for cleanup process "
+                      << atf::text::to_string(pid) << ": "
+                      << ::strerror(errno) << std::endl;
+        if (!WIFEXITED(status) || WEXITSTATUS(status) != EXIT_SUCCESS)
+            std::cerr << "WARNING: Cleanup process ended unexpectedly"
+                      << std::endl;
+    }
+}
+
 impl::tcr
 impl::tc::run(void)
     const
 {
-    assert(!m_meta_data.empty());
+    PRE(!m_meta_data.empty());
 
     tcr tcr;
 
@@ -652,7 +665,7 @@ void
 impl::tc::require_prog(const std::string& prog)
     const
 {
-    assert(!prog.empty());
+    PRE(!prog.empty());
 
     fs::path p(prog);
 
@@ -661,8 +674,8 @@ impl::tc::require_prog(const std::string& prog)
             throw impl::tcr::skipped("The required program " + prog +
                                      " could not be found");
     } else {
-        assert(p.branch_path() == fs::path("."));
-        if (fs::find_prog_in_path(prog) == fs::path("."))
+        INV(p.branch_path() == fs::path("."));
+        if (fs::find_prog_in_path(prog).empty())
             throw impl::tcr::skipped("The required program " + prog +
                                      " could not be found in the PATH");
     }
@@ -687,8 +700,6 @@ private:
 
     atf::tests::vars_map m_vars;
 
-    static atf::tests::vars_map::value_type parse_var(const std::string&);
-
     std::string specific_args(void) const;
     options_set specific_options(void) const;
     void process_option(int, const char*);
@@ -766,31 +777,14 @@ tp::process_option(int ch, const char* arg)
 
     case 'v':
         {
-            atf::tests::vars_map::value_type v = parse_var(arg);
+            atf::tests::vars_map::value_type v =
+                atf::tests::vars_map::parse(arg);
             m_vars[v.first] = v.second;
         }
         break;
 
     default:
-        assert(false);
-    }
-}
-
-atf::tests::vars_map::value_type
-tp::parse_var(const std::string& str)
-{
-    if (str.empty())
-        throw std::runtime_error("-v requires a non-empty argument");
-
-    std::vector< std::string > ws = atf::text::split(str, "=");
-    if (ws.size() == 1 && str[str.length() - 1] == '=') {
-        return atf::tests::vars_map::value_type(ws[0], "");
-    } else {
-        if (ws.size() != 2)
-            throw std::runtime_error("-v requires an argument of the form "
-                                     "var=value");
-
-        return atf::tests::vars_map::value_type(ws[0], ws[1]);
+        UNREACHABLE;
     }
 }
 
@@ -868,7 +862,7 @@ tp::filter_tcs(tc_vector tcs, const std::set< std::string >& tcnames)
 
             tc_vector::iterator tciter =
                 std::find_if(tcs.begin(), tcs.end(), tc_equal_to_ident(name));
-            assert(tciter != tcs.end());
+            INV(tciter != tcs.end());
             tcso.push_back(*tciter);
         }
     }
@@ -929,7 +923,8 @@ tp::run_tcs(void)
     signal_holder sigint(SIGINT);
     signal_holder sigterm(SIGTERM);
 
-    atf::formats::atf_tcs_writer w(results_stream(), tcs.size());
+    atf::formats::atf_tcs_writer w(results_stream(), std::cout, std::cerr,
+                                   tcs.size());
     for (tc_vector::iterator iter = tcs.begin();
          iter != tcs.end(); iter++) {
         impl::tc* tc = *iter;
