@@ -58,6 +58,8 @@ public:
     bool get_bool(const std::string&, bool) const;
 
     bool has(const std::string&) const;
+
+    static value_type parse(const std::string&);
 };
 
 // ------------------------------------------------------------------------
@@ -111,6 +113,9 @@ class tc {
 
     tcr safe_run(void) const;
     tcr fork_body(const std::string&) const;
+    void fork_cleanup(const std::string&) const;
+
+    void check_requirements(void) const;
 
 protected:
     virtual void head(void) = 0;
