@@ -57,134 +57,6 @@ ident_body()
     done
 }
 
-# -------------------------------------------------------------------------
-# Tests for the "isolated" meta-data property.
-# -------------------------------------------------------------------------
-
-atf_test_case isolated_default
-isolated_default_head()
-{
-    atf_set "descr" "Tests that the 'isolated' test property works" \
-                    "when the user does not override the work directory"
-}
-isolated_default_body()
-{
-    srcdir=$(atf_get_srcdir)
-    h_cpp=${srcdir}/h_cpp
-    h_sh=${srcdir}/h_sh
-    tmpdir=$(cd $(atf-config -t atf_workdir) && pwd -P)
-
-    for h in ${h_cpp} ${h_sh}; do
-        atf_check "${h} -s ${srcdir} -v isolated=no \
-                   -v pathfile=$(pwd)/expout isolated_path" 0 ignore ignore
-        atf_check "pwd -P" 0 expout null
-
-        atf_check "${h} -s ${srcdir} -v isolated=yes \
-                  -v pathfile=$(pwd)/path isolated_path" 0 ignore ignore
-        atf_check "grep '^${tmpdir}/atf.' <path" 0 ignore ignore
-    done
-}
-
-atf_test_case isolated_tmpdir
-isolated_tmpdir_head()
-{
-    atf_set "descr" "Tests that the 'isolated' test property works" \
-                    "when the user overrides the work directory through" \
-                    "the ATF_WORKDIR environment variable"
-}
-isolated_tmpdir_body()
-{
-    srcdir=$(atf_get_srcdir)
-    h_cpp=${srcdir}/h_cpp
-    h_sh=${srcdir}/h_sh
-    tmpdir=$(pwd -P)/workdir
-    mkdir ${tmpdir}
-
-    for h in ${h_cpp} ${h_sh}; do
-        atf_check "ATF_WORKDIR=${tmpdir} ${h} -s ${srcdir} -v isolated=no \
-                   -v pathfile=$(pwd)/expout isolated_path" 0 ignore ignore
-        atf_check "pwd -P" 0 expout null
-
-        atf_check "ATF_WORKDIR=${tmpdir} ${h} -s ${srcdir} -v isolated=yes \
-                   -v pathfile=$(pwd)/path isolated_path" 0 ignore ignore
-        atf_check "grep '^${tmpdir}/atf.' <path" 0 ignore ignore
-    done
-}
-
-atf_test_case isolated_conf
-isolated_conf_head()
-{
-    atf_set "descr" "Tests that the 'isolated' test property works" \
-                    "when the user overrides the work directory through" \
-                    "the test program's 'workdir' configuration option"
-}
-isolated_conf_body()
-{
-    srcdir=$(atf_get_srcdir)
-    h_cpp=${srcdir}/h_cpp
-    h_sh=${srcdir}/h_sh
-    tmpdir=$(pwd -P)/workdir
-    mkdir ${tmpdir}
-
-    for h in ${h_cpp} ${h_sh}; do
-        atf_check "${h} -s ${srcdir} -v isolated=no \
-                   -v pathfile=$(pwd)/expout -v workdir=${tmpdir} \
-                   isolated_path" 0 ignore ignore
-        atf_check "pwd -P" 0 expout null
-
-        atf_check "${h} -s ${srcdir} -v isolated=yes \
-                   -v pathfile=$(pwd)/path -v workdir=${tmpdir} \
-                   isolated_path" 0 ignore ignore
-        atf_check "grep '^${tmpdir}/atf.' <path" 0 ignore null
-    done
-}
-
-check_emptydir()
-{
-    dir=${1} what=${2}
-    set -- ${dir}/atf.*
-    [ -f ${1} ] && \
-        atf_fail "${what} did not properly remove all temporary files"
-}
-
-atf_test_case isolated_cleanup
-isolated_cleanup_head()
-{
-    atf_set "descr" "Tests that the directories used to isolate test" \
-                    "cases are properly cleaned up once the test case" \
-                    "exits"
-}
-isolated_cleanup_body()
-{
-    srcdir=$(atf_get_srcdir)
-    h_cpp=${srcdir}/h_cpp
-    h_sh=${srcdir}/h_sh
-    tmpdir=$(pwd -P)/workdir
-    mkdir ${tmpdir}
-
-    for h in ${h_cpp} ${h_sh}; do
-        # First try to clean a work directory that, supposedly, does not
-        # have any subdirectories.  Do this for both isolated set to yes
-        # and no, as there used to be a bug that make the latter case
-        # leave a temporary file behind.
-        atf_check "${h} -s ${srcdir} -v isolated=no \
-                   -v pathfile=$(pwd)/path -v workdir=${tmpdir} \
-                   isolated_path" 0 ignore ignore
-        check_emptydir ${tmpdir} isolated=no
-        atf_check "${h} -s ${srcdir} -v isolated=yes \
-                   -v pathfile=$(pwd)/path -v workdir=${tmpdir} \
-                   isolated_path" 0 ignore ignore
-        atf_check "test -d $(cat path)" 1 null null
-        check_emptydir ${tmpdir} isolated=yes
-
-        # Now do the same but with a work directory that has subdirectories.
-        # The program will have to recurse into them to clean them all.
-        atf_check "${h} -s ${srcdir} -v pathfile=$(pwd)/path \
-                   -v workdir=${tmpdir} isolated_cleanup" 0 ignore ignore
-        atf_check "test -d $(cat path)" 1 null null
-    done
-}
-
 # -------------------------------------------------------------------------
 # Tests for the "require_config" meta-data property.
 # -------------------------------------------------------------------------
@@ -397,10 +269,6 @@ require_user_multiple_body()
 atf_init_test_cases()
 {
     atf_add_test_case ident
-    atf_add_test_case isolated_default
-    atf_add_test_case isolated_tmpdir
-    atf_add_test_case isolated_conf
-    atf_add_test_case isolated_cleanup
     atf_add_test_case require_config
     atf_add_test_case require_progs_func
     atf_add_test_case require_progs_header
