@@ -39,13 +39,13 @@ extern "C" {
 #include <sys/stat.h>
 }
 
-#include <cassert>
 #include <cstdlib>
 #include <fstream>
 #include <iostream>
 
 #include "atf/application.hpp"
 #include "atf/config.hpp"
+#include "atf/sanity.hpp"
 
 static
 void
@@ -98,8 +98,7 @@ atf_compile::specific_options(void)
     const
 {
     options_set opts;
-    opts.insert(option('o', "out-file",
-                       "Write output to the given file"));
+    opts.insert(option('o', "out-file", "Name of the output file"));
     return opts;
 }
 
@@ -112,7 +111,7 @@ atf_compile::process_option(int ch, const char* arg)
         break;
 
     default:
-        assert(false);
+        UNREACHABLE;
     }
 }
 
@@ -140,19 +139,23 @@ atf_compile::main(void)
     if (m_argc < 1)
         throw atf::usage_error("No test program specified");
 
-    if (m_outfile.empty()) {
-        compile(std::cout);
-    } else {
-        std::ofstream os(m_outfile.c_str());
-        if (!os)
-            throw std::runtime_error("Cannot open output file `" +
-                                     m_outfile + "'");
-        compile(os);
-        os.close();
+    if (m_outfile.empty())
+        throw atf::usage_error("No output file specified");
 
-        // XXX Handle errors and respect umask.
-        ::chmod(m_outfile.c_str(), 0755);
-    }
+    std::ofstream os(m_outfile.c_str());
+    if (!os)
+        throw std::runtime_error("Cannot open output file `" +
+                                 m_outfile + "'");
+    compile(os);
+    os.close();
+
+    mode_t umask = ::umask(S_IRWXU | S_IRWXG | S_IRWXO);
+    ::umask(umask);
+
+    if (::chmod(m_outfile.c_str(),
+                (S_IRWXU | S_IRWXG | S_IRWXO) & ~umask) == -1)
+        throw std::runtime_error("Cannot set executable permissions "
+                                 "on `" + m_outfile + "'");
 
     return EXIT_SUCCESS;
 }
