@@ -34,7 +34,6 @@
 // IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 //
 
-#include <cassert>
 #include <fstream>
 #include <iostream>
 #include <memory>
@@ -44,6 +43,7 @@
 #include "atf/application.hpp"
 #include "atf/fs.hpp"
 #include "atf/formats.hpp"
+#include "atf/sanity.hpp"
 #include "atf/text.hpp"
 #include "atf/ui.hpp"
 
@@ -82,6 +82,7 @@ public:
     writer(void) {}
     virtual ~writer(void) {}
 
+    virtual void write_info(const std::string&, const std::string&) {}
     virtual void write_ntps(size_t) {}
     virtual void write_tp_start(const std::string&, size_t) {}
     virtual void write_tp_end(const std::string&) {}
@@ -106,6 +107,7 @@ public:
 //!
 class csv_writer : public writer {
     ostream_ptr m_os;
+    bool m_failed;
 
     std::string m_tpname;
     std::string m_tcname;
@@ -121,13 +123,20 @@ public:
     write_tp_start(const std::string& name, size_t ntcs)
     {
         m_tpname = name;
+        m_failed = false;
     }
 
     virtual
     void
     write_tp_end(const std::string& reason)
     {
-        // TODO: Report failed test programs in some way.
+        if (!reason.empty())
+            (*m_os) << "tp, " << m_tpname << ", bogus, " << reason
+                    << std::endl;
+        else if (m_failed)
+            (*m_os) << "tp, " << m_tpname << ", failed" << std::endl;
+        else
+            (*m_os) << "tp, " << m_tpname << ", passed" << std::endl;
     }
 
     virtual
@@ -141,17 +150,18 @@ public:
     void
     write_tc_end(const atf::tests::tcr& tcr)
     {
-        std::string str;
+        std::string str = "tc, ";
         if (tcr.get_status() == atf::tests::tcr::status_passed) {
-            str = m_tpname + ", " + m_tcname + ", passed";
+            str += m_tpname + ", " + m_tcname + ", passed";
         } else if (tcr.get_status() == atf::tests::tcr::status_failed) {
-            str = m_tpname + ", " + m_tcname + ", failed, " +
-                  tcr.get_reason();
+            str += m_tpname + ", " + m_tcname + ", failed, " +
+                   tcr.get_reason();
+            m_failed = true;
         } else if (tcr.get_status() == atf::tests::tcr::status_skipped) {
-            str = m_tpname + ", " + m_tcname + ", skipped, " +
-                  tcr.get_reason();
+            str += m_tpname + ", " + m_tcname + ", skipped, " +
+                   tcr.get_reason();
         } else
-            assert(false);
+            UNREACHABLE;
         (*m_os) << str << std::endl;
     }
 };
@@ -177,6 +187,15 @@ class ticker_writer : public writer {
     std::vector< std::string > m_failed_tcs;
     std::vector< std::string > m_failed_tps;
 
+    void
+    write_info(const std::string& what, const std::string& val)
+    {
+        if (what == "tests.root") {
+            (*m_os) << "Tests root: " << val << std::endl
+                    << std::endl;
+        }
+    }
+
     void
     write_ntps(size_t ntps)
     {
@@ -249,7 +268,7 @@ class ticker_writer : public writer {
             str = "Skipped: " + tcr.get_reason();
             m_tcs_skipped++;
         } else
-            assert(false);
+            UNREACHABLE;
 
         // XXX Wrap text.  format_text_with_tag does not currently allow
         // to specify the current column, which is needed because we have
@@ -306,6 +325,129 @@ public:
     }
 };
 
+// ------------------------------------------------------------------------
+// The "xml" class.
+// ------------------------------------------------------------------------
+
+//!
+//! \brief A single-file XML output format.
+//!
+//! The xml_writer class implements a formatter that prints the results
+//! of test cases in an XML format easily parseable later on by other
+//! utilities.
+//!
+class xml_writer : public writer {
+    ostream_ptr m_os;
+
+    size_t m_curtp, m_ntps;
+    size_t m_tcs_passed, m_tcs_failed, m_tcs_skipped;
+    std::string m_tcname, m_tpname;
+    std::vector< std::string > m_failed_tcs;
+    std::vector< std::string > m_failed_tps;
+
+    static
+    std::string
+    attrval(const std::string& str)
+    {
+        return str;
+    }
+
+    static
+    std::string
+    elemval(const std::string& str)
+    {
+        std::string ostr;
+        for (std::string::const_iterator iter = str.begin();
+             iter != str.end(); iter++) {
+            switch (*iter) {
+            case '&': ostr += "&amp;"; break;
+            case '<': ostr += "&lt;"; break;
+            case '>': ostr += "&gt;"; break;
+            default:  ostr += *iter;
+            }
+        }
+        return ostr;
+    }
+
+    void
+    write_info(const std::string& what, const std::string& val)
+    {
+        (*m_os) << "<info class=\"" << what << "\">" << val << "</info>"
+                << std::endl;
+    }
+
+    void
+    write_tp_start(const std::string& tp, size_t ntcs)
+    {
+        (*m_os) << "<tp id=\"" << attrval(tp) << "\">" << std::endl;
+    }
+
+    void
+    write_tp_end(const std::string& reason)
+    {
+        if (!reason.empty())
+            (*m_os) << "<failed>" << elemval(reason) << "</failed>"
+                    << std::endl;
+        (*m_os) << "</tp>" << std::endl;
+    }
+
+    void
+    write_tc_start(const std::string& tcname)
+    {
+        (*m_os) << "<tc id=\"" << attrval(tcname) << "\">" << std::endl;
+    }
+
+    void
+    write_tc_stdout_line(const std::string& line)
+    {
+        (*m_os) << "<so>" << elemval(line) << "</so>" << std::endl;
+    }
+
+    void
+    write_tc_stderr_line(const std::string& line)
+    {
+        (*m_os) << "<se>" << elemval(line) << "</se>" << std::endl;
+    }
+
+    void
+    write_tc_end(const atf::tests::tcr& tcr)
+    {
+        std::string str;
+
+        atf::tests::tcr::status s = tcr.get_status();
+        if (s == atf::tests::tcr::status_passed) {
+            (*m_os) << "<passed />" << std::endl;
+        } else if (s == atf::tests::tcr::status_failed) {
+            (*m_os) << "<failed>" << elemval(tcr.get_reason())
+                    << "</failed>" << std::endl;
+        } else if (s == atf::tests::tcr::status_skipped) {
+            (*m_os) << "<skipped>" << elemval(tcr.get_reason())
+                    << "</skipped>" << std::endl;
+        } else
+            UNREACHABLE;
+        (*m_os) << "</tc>" << std::endl;
+    }
+
+    void
+    write_eof(void)
+    {
+        (*m_os) << "</tests-results>" << std::endl;
+    }
+
+public:
+    xml_writer(const atf::fs::path& p) :
+        m_os(open_outfile(p))
+    {
+        (*m_os) << "<?xml version=\"1.0\"?>" << std::endl
+                << "<!DOCTYPE tests-results PUBLIC "
+                   "\"-//NetBSD//DTD ATF Tests Results 0.1//EN\" "
+                   "\"http://www.NetBSD.org/XML/atf/tests-results.dtd\">"
+                << std::endl
+                << std::endl
+                << "<tests-results>" << std::endl;
+    }
+};
+
 // ------------------------------------------------------------------------
 // The "converter" class.
 // ------------------------------------------------------------------------
@@ -321,6 +463,14 @@ class converter : public atf::formats::atf_tps_reader {
     typedef std::vector< writer* > outs_vector;
     outs_vector m_outs;
 
+    void
+    got_info(const std::string& what, const std::string& val)
+    {
+        for (outs_vector::iterator iter = m_outs.begin();
+             iter != m_outs.end(); iter++)
+            (*iter)->write_info(what, val);
+    }
+
     void
     got_ntps(size_t ntps)
     {
@@ -353,6 +503,22 @@ class converter : public atf::formats::atf_tps_reader {
             (*iter)->write_tc_start(tcname);
     }
 
+    void
+    got_tc_stdout_line(const std::string& line)
+    {
+        for (outs_vector::iterator iter = m_outs.begin();
+             iter != m_outs.end(); iter++)
+            (*iter)->write_tc_stdout_line(line);
+    }
+
+    void
+    got_tc_stderr_line(const std::string& line)
+    {
+        for (outs_vector::iterator iter = m_outs.begin();
+             iter != m_outs.end(); iter++)
+            (*iter)->write_tc_stderr_line(line);
+    }
+
     void
     got_tc_end(const atf::tests::tcr& tcr)
     {
@@ -389,6 +555,8 @@ public:
             m_outs.push_back(new csv_writer(p));
         } else if (fmt == "ticker") {
             m_outs.push_back(new ticker_writer(p));
+        } else if (fmt == "xml") {
+            m_outs.push_back(new xml_writer(p));
         } else
             throw std::runtime_error("Unknown format `" + fmt + "'");
     }
@@ -441,7 +609,7 @@ atf_report::process_option(int ch, const char* arg)
         break;
 
     default:
-        assert(false);
+        UNREACHABLE;
     }
 }
 
