@@ -34,13 +34,16 @@
 // IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 //
 
+#if defined(HAVE_CONFIG_H)
+#include "config.h"
+#endif
+
 extern "C" {
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <unistd.h>
 }
 
-#include <cassert>
 #include <cerrno>
 #include <cstdlib>
 #include <fstream>
@@ -57,6 +60,7 @@ extern "C" {
 #include "atf/fs.hpp"
 #include "atf/io.hpp"
 #include "atf/parser.hpp"
+#include "atf/sanity.hpp"
 #include "atf/tests.hpp"
 #include "atf/text.hpp"
 
@@ -87,7 +91,7 @@ class muxer : public atf::formats::atf_tcs_reader {
     atf::fs::path m_tp;
     atf::formats::atf_tps_writer m_writer;
 
-    bool m_finalized;
+    bool m_inited, m_finalized;
     size_t m_ntcs;
     std::string m_tcname;
 
@@ -98,9 +102,10 @@ class muxer : public atf::formats::atf_tcs_reader {
     got_ntcs(size_t ntcs)
     {
         m_writer.start_tp(m_tp.str(), ntcs);
+        m_inited = true;
         if (ntcs == 0)
-            throw atf::serial::format_error("Bogus test program: reported "
-                                            "0 test cases");
+            throw atf::formats::format_error("Bogus test program: reported "
+                                             "0 test cases");
     }
 
     void
@@ -120,9 +125,8 @@ class muxer : public atf::formats::atf_tcs_reader {
             m_skipped++;
         } else if (s == atf::tests::tcr::status_failed) {
             m_failed++;
-        } else {
-            assert(false);
-        }
+        } else
+            UNREACHABLE;
 
         m_writer.end_tc(tcr);
         m_tcname = "";
@@ -146,6 +150,7 @@ public:
         atf::formats::atf_tcs_reader(is),
         m_tp(tp),
         m_writer(w),
+        m_inited(false),
         m_finalized(false),
         m_passed(0),
         m_failed(0),
@@ -163,19 +168,24 @@ public:
     void
     finalize(const std::string& reason = "")
     {
+        PRE(!m_finalized);
+
+        if (!m_inited)
+            m_writer.start_tp(m_tp.str(), 0);
         if (!m_tcname.empty()) {
-            assert(!reason.empty());
+            INV(!reason.empty());
             got_tc_end(atf::tests::tcr::failed("Bogus test program"));
         }
 
-        assert(!m_finalized);
         m_writer.end_tp(reason);
         m_finalized = true;
     }
 
     ~muxer(void)
     {
-        assert(m_finalized);
+        // The following is incorrect because we cannot throw an exception
+        // from a destructor.  Let's just hope that this never happens.
+        PRE(m_finalized);
     }
 };
 
@@ -240,38 +250,20 @@ atf_run::atf_run(void) :
 {
 }
 
-// XXX Duplicate from atf/tests.cpp.
-atf::tests::vars_map::value_type
-atf_run::parse_var(const std::string& str)
-{
-    if (str.empty())
-        throw std::runtime_error("-v requires a non-empty argument");
-
-    std::vector< std::string > ws = atf::text::split(str, "=");
-    if (ws.size() == 1 && str[str.length() - 1] == '=') {
-        return atf::tests::vars_map::value_type(ws[0], "");
-    } else {
-        if (ws.size() != 2)
-            throw std::runtime_error("-v requires an argument of the form "
-                                     "var=value");
-
-        return atf::tests::vars_map::value_type(ws[0], ws[1]);
-    }
-}
-
 void
 atf_run::process_option(int ch, const char* arg)
 {
     switch (ch) {
     case 'v':
         {
-            atf::tests::vars_map::value_type v = parse_var(arg);
+            atf::tests::vars_map::value_type v =
+                atf::tests::vars_map::parse(arg);
             m_cmdline_vars[v.first] = v.second;
         }
         break;
 
     default:
-        assert(false);
+        UNREACHABLE;
     }
 }
 
@@ -318,7 +310,7 @@ atf_run::run_test_directory(const atf::fs::path& tp,
     {
         atf::tests::vars_map::const_iterator iter =
             af.props().find("test-suite");
-        assert(iter != af.props().end());
+        INV(iter != af.props().end());
         read_config((*iter).second);
     }
 
@@ -436,70 +428,68 @@ atf_run::run_test_program_parent(const atf::fs::path& tp,
 
     // Process the test case's output and multiplex it into our output
     // stream as we read it.
+    muxer m(tp, w, resin);
+    std::string fmterr;
     try {
-        muxer m(tp, w, resin);
-        std::string fmterr;
-        try {
-            m.read(outin, errin);
-        } catch (const atf::parser::parse_errors& e) {
-            fmterr = e.what();
-        } catch (const atf::serial::format_error& e) {
-            fmterr = e.what();
+        m.read(outin, errin);
+    } catch (const atf::parser::parse_errors& e) {
+        fmterr = "There were errors parsing the output of the test "
+                 "program:";
+        for (atf::parser::parse_errors::const_iterator iter = e.begin();
+             iter != e.end(); iter++) {
+            fmterr += " Line " + atf::text::to_string((*iter).first) +
+                      ": " + (*iter).second + ".";
         }
-
-        outin.close();
-        errin.close();
-        resin.close();
-
-        int code, status;
-        if (::waitpid(pid, &status, 0) == -1) {
-            m.finalize("waitpid(2) on the child process " +
-                       atf::text::to_string(pid) + " failed" +
-                       (fmterr.empty() ? "" : (".  " + fmterr)));
-            code = EXIT_FAILURE;
-        } else {
-            if (WIFEXITED(status)) {
-                code = WEXITSTATUS(status);
-                if (m.failed() > 0 && code == EXIT_SUCCESS) {
-                    code = EXIT_FAILURE;
-                    m.finalize("Test program returned success but some test "
-                               "cases failed" +
-                               (fmterr.empty() ? "" : (".  " + fmterr)));
-                } else {
-                    code = fmterr.empty() ? code : EXIT_FAILURE;
-                    m.finalize(fmterr);
-                }
-            } else if (WIFSIGNALED(status)) {
-                code = EXIT_FAILURE;
-                m.finalize("Test program received signal " +
-                           atf::text::to_string(WTERMSIG(status)) +
-                           (WCOREDUMP(status) ? " (core dumped)" : "") +
-                           (fmterr.empty() ? "" : (".  " + fmterr)));
-            } else if (WIFSTOPPED(status)) {
-                code = EXIT_FAILURE;
-                m.finalize("Test program stopped due to signal " +
-                           atf::text::to_string(WSTOPSIG(status)) +
-                           (fmterr.empty() ? "" : (".  " + fmterr)));
-            } else
-                throw std::runtime_error
-                    ("Child process " + atf::text::to_string(pid) +
-                     " terminated with an unknown status condition " +
-                     atf::text::to_string(status));
-        }
-        return code;
+    } catch (const atf::formats::format_error& e) {
+        fmterr = e.what();
     } catch (...) {
-        // XXX This catch block is here to prevent a SIGPIPE in FreeBSD 6.2
-        // when the construction of the muxer fails.  This should be handled
-        // by the other try block inside this one, but at the moment we do
-        // some reads straight from the constructor which is proving to be
-        // a very annoying approach.
+        UNREACHABLE;
+    }
+
+    try {
         outin.close();
         errin.close();
         resin.close();
-        int status;
-        (void)::waitpid(pid, &status, 0);
-        throw;
+    } catch (...) {
+        UNREACHABLE;
     }
+
+    int code, status;
+    if (::waitpid(pid, &status, 0) == -1) {
+        m.finalize("waitpid(2) on the child process " +
+                   atf::text::to_string(pid) + " failed" +
+                   (fmterr.empty() ? "" : (".  " + fmterr)));
+        code = EXIT_FAILURE;
+    } else {
+        if (WIFEXITED(status)) {
+            code = WEXITSTATUS(status);
+            if (m.failed() > 0 && code == EXIT_SUCCESS) {
+                code = EXIT_FAILURE;
+                m.finalize("Test program returned success but some test "
+                           "cases failed" +
+                           (fmterr.empty() ? "" : (".  " + fmterr)));
+            } else {
+                code = fmterr.empty() ? code : EXIT_FAILURE;
+                m.finalize(fmterr);
+            }
+        } else if (WIFSIGNALED(status)) {
+            code = EXIT_FAILURE;
+            m.finalize("Test program received signal " +
+                       atf::text::to_string(WTERMSIG(status)) +
+                       (WCOREDUMP(status) ? " (core dumped)" : "") +
+                       (fmterr.empty() ? "" : (".  " + fmterr)));
+        } else if (WIFSTOPPED(status)) {
+            code = EXIT_FAILURE;
+            m.finalize("Test program stopped due to signal " +
+                       atf::text::to_string(WSTOPSIG(status)) +
+                       (fmterr.empty() ? "" : (".  " + fmterr)));
+        } else
+            throw std::runtime_error
+                ("Child process " + atf::text::to_string(pid) +
+                 " terminated with an unknown status condition " +
+                 atf::text::to_string(status));
+    }
+    return code;
 }
 
 int
@@ -515,24 +505,10 @@ atf_run::run_test_program(const atf::fs::path& tp,
                                 "fork(2) failed", errno);
     } else if (pid == 0) {
         run_test_program_child(tp, outpipe, errpipe, respipe);
-        // NOTREACHED
-        assert(false);
-        errcode = EXIT_FAILURE;
+        UNREACHABLE;
     } else {
-        try {
-            errcode = run_test_program_parent(tp, w, outpipe, errpipe,
-                                              respipe, pid);
-        } catch (const atf::parser::parse_errors& e) {
-            // Parse errors captured here were caused by errors in the
-            // child's header, which were not handled by the function.
-            std::string fmterr = e.what();
-            for (std::string::size_type i = 0; i < fmterr.length(); i++)
-                if (fmterr[i] == '\n' || fmterr[i] == '\r')
-                    fmterr[i] = ' ';
-            w.start_tp(tp.str(), 0);
-            w.end_tp(fmterr);
-            errcode = EXIT_FAILURE;
-        }
+        errcode = run_test_program_parent(tp, w, outpipe, errpipe,
+                                          respipe, pid);
     }
 
     return errcode;
@@ -590,6 +566,17 @@ atf_run::read_config(const std::string& name)
     }
 }
 
+static
+void
+call_hook(const std::string& tool, const std::string& hook)
+{
+    std::string sh = atf::config::get("atf_shell");
+    atf::fs::path p = atf::fs::path(atf::config::get("atf_pkgdatadir")) /
+                      (tool + ".hooks");
+    std::string cmd = sh + " '" + p.str() + "' '" + hook + "'";
+    std::system(cmd.c_str());
+}
+
 int
 atf_run::main(void)
 {
@@ -610,17 +597,21 @@ atf_run::main(void)
     {
         atf::tests::vars_map::const_iterator iter =
             af.props().find("test-suite");
-        assert(iter != af.props().end());
+        INV(iter != af.props().end());
         read_config((*iter).second);
     }
 
-    atf::formats::atf_tps_writer w(std::cout, count_tps(tps));
+    atf::formats::atf_tps_writer w(std::cout);
+    call_hook("atf-run", "info_start_hook");
+    w.ntps(count_tps(tps));
 
     bool ok = true;
     for (std::vector< std::string >::const_iterator iter = tps.begin();
          iter != tps.end(); iter++)
         ok &= (run_test(atf::fs::path(*iter), w) == EXIT_SUCCESS);
 
+    call_hook("atf-run", "info_end_hook");
+
     return ok ? EXIT_SUCCESS : EXIT_FAILURE;
 }
 
